<?php
/* This file is part of BBClone (The PHP web counter on steroids)
 *
 * $Header: /cvs/bbclone/lib/io.php,v 1.42 2004/06/20 17:22:01 joku Exp $
 *
 * Copyright (C) 2001-2004, the BBClone Team (see file doc/authors.txt
 * distributed with this library)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See doc/copying.txt for details
 */

//initialize the bbc_marker class
function bbc_exec_marker($version) {
  $str = "\n<!-- BBClone v $version ";
  $bbc_marker =& new bbc_marker;

  if ($bbc_marker->ignored === true) return $str."client ignored -->\n";
  else $msg =& $bbc_marker->bbc_write_entry();

  switch ($msg[1]) {
    case "ok":
      $str .= "OK: counter files updated";
      if (!defined("_OK")) define("_OK", 1);
      break;
    case "err_lock":
      $str .= "KO: unable to lock ".$msg[0]." for writing";
      break;
    case "err_wri":
      $str .= "KO: ".$msg[0]." is read-only";
      break;
    case "err_acc":
      $str .= "KO: ".$msg[0]." is inaccessible";
      break;
    default:
      $str .= "KO: unknown error with ".$msg[0];
  }
  return $str." -->\n";
}

// kill stats if desired
function bbc_kill_stats() {
  global $BBC_ACCESS_FILE, $BBC_CACHE_PATH, $BBC_COUNTER_FILES, $BBC_COUNTER_PREFIX, $BBC_COUNTER_SUFFIX,
         $BBC_NO_STRING, $BBC_LAST_FILE;

  for ($i = 0; $i < $BBC_COUNTER_FILES; $i++) {
    $file = $BBC_CACHE_PATH.$BBC_COUNTER_PREFIX.$i.$BBC_COUNTER_SUFFIX;

    fclose(fopen($file, "wb"));
  }

  fclose(fopen($BBC_ACCESS_FILE, "wb"));
  fclose(fopen($BBC_LAST_FILE, "wb"));
}

// Parse all counter files of var/ and return an array of N rows,
// with N as amount of new connections, sorted in increasing time of connection.
// The counters files are emptied afterwards.
function bbc_counter_to_array() {
  global $BBC_CACHE_PATH, $BBC_COUNTER_PREFIX, $BBC_COUNTER_SUFFIX, $BBC_SEP, $BBC_COUNTER_COLUMNS,
         $BBC_COUNTER_COLUMN_NAMES, $BBC_COUNTER_FILES;

  for ($i = 0, $nb_new_entry = 0; $i < $BBC_COUNTER_FILES; $i++) {
    $file = $BBC_CACHE_PATH.$BBC_COUNTER_PREFIX.$i.$BBC_COUNTER_SUFFIX;

    if (!is_readable($file)) {
      bbc_err_msg($file);
      $no_acc = 1;
      continue;
    }

    $fp = fopen($file, "r");

    while (($line = fgetcsv($fp, 4096, $BBC_SEP)) !== false) {
      for ($j = 0, $max = count($line); $j < $max; $j++) $line[$j] = trim($line[$j]);
      // Avoiding ill formed counter file
      if ((empty($line[0])) || (!preg_match(":^[0-9]+$:",$line[0]))) continue;

      for ($k = 0; ($k < $BBC_COUNTER_COLUMNS); $k++) {
        $counter_array[$nb_new_entry][($BBC_COUNTER_COLUMN_NAMES[$k])] = $line[$k];
      }
      $nb_new_entry++;
    }
    fclose($fp);

    // reset the counter files
    if (is_writable($file)) fclose(fopen($file, "wb"));
    else (empty($no_acc) ? bbc_err_msg($file, "write") : "");
  }
  if (!empty($counter_array)) usort($counter_array, "bbc_sort_time_sc");
  return (empty($counter_array) ? array() : $counter_array);
}

function bbc_array_to_str(&$tab) {
// This function return a string description of an array.
// Format (_ == space):
// |_array(
// |__key1 => scal1, key2 => scal2, ...
// |__key3 =>
// |___array(
// |____key1 => scal1, ...
// |___),
// |__key4 => scal4, ...
// |_)

  static $indent = "";

  $oldindent = $indent;
  $indent   .= "  ";
  $sep       = "";

  $str = $indent."array(\n";
  $last_is_array = false;
  $k = 0;

  reset($tab);

  while (list($key, $val) = each($tab)) {
    // The separator treatment
    if (($last_is_array) || (is_array($val) && ($k !== 0))) {
      $str .= $sep."\n";
    }
    else $str .= $sep;

      // The key treatment
    if (preg_match(":^[0-9]+$:", $key)) {
      if ($key !== $k) {
        $str .= (((is_array($val)) || ($k === 0) || ($last_is_array)) ? "$indent  " : "")
               ."$key =>".((is_array($val)) ? "\n" : " ");
      }
      else $str .= ($k === 0) ? (is_array($val) ? "" : "$indent  ") : "";
    }
    else {
      $str .= (((is_array($val)) || ($k === 0) || ($last_is_array)) ? "$indent  " : "")
             ."\"$key\" =>".((is_array($val)) ? "\n" : " ");
    }

    // The value treatment
    $last_is_array = false;
    if (is_array($val)) {
      $str .= bbc_array_to_str($val);
      $last_is_array = true;
      $sep = ",";
    }
    else {
      $str .= (preg_match(":^[0-9]+$:", $val) ? $val : "\"$val\"");
      $sep = ", ";
    }
    $k++;
  }
  $str .= "\n$indent)";
  $indent = $oldindent;
  return $str;
}

// write data, returns file pointer on success else false
function bbc_begin_write($file, $data) {
  $fp = fopen($file, "r+");

  if (flock($fp, LOCK_EX)) {
    rewind($fp);
    fwrite($fp, $data);
    fflush($fp);
    ftruncate($fp, ftell($fp));
    return($fp);
  }
  else {
    fclose($fp);
    return false;
  }
}

// finish writing to a file
function bbc_end_write($fp) {
  if ($fp === false) return false;

  flock($fp, LOCK_UN);
  fclose($fp);
  return true;
}
?>