<?php
/* This file is part of BBClone (A PHP web counter on steroids)
 *
 * $Header: /cvs/bbclone/lib/marker.php,v 1.27 2004/08/13 21:32:28 olliver Exp $
 *
 * Copyright (C) 2001-2004, the BBClone Team (see file doc/authors.txt
 * distributed with this library)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See doc/copying.txt for details
 */

// Main Class Counter
class bbc_marker {
  var $sep, $filename, $ignored, $string;

  // remove unwanted stuff from user input
  function bbc_clean($input) {
    $input = (($sp = strpos($input, $this->sep)) !== false) ? substr($input, 0, $sp) : $input;
    $input = str_replace("\\", "/", stripslashes($input));
    $input = trim(htmlspecialchars(strip_tags($input), ENT_QUOTES));

    // Limit the maximum length to 512 chars
    return substr(str_replace("$", "&#36;", $input), 0, 512);
  }

  // randomly choose a counter file to write to
  function bbc_counter_file($cache_path, $counter_pre, $counter_suf) {
    global $BBC_COUNTER_FILES;

    mt_srand((double) microtime() * 1000000);
    return ($cache_path.$counter_pre.mt_rand(0, ($BBC_COUNTER_FILES - 1)).$counter_suf);
  }

  // validates a hostname or ip address
  function bbc_valid_ip($srv, $prx = 0) {
    $iptest = explode(".", $srv);
    $iptest = defined("_rev") ? array_reverse($iptest) : $iptest;
    $oct = count($iptest);
    // reserved class A ranges. A public IP must not start with one of these
    $reserved = array(
      1, 2, 5, 10, 14, 22, 23, 27, 31, 36, 37, 39, 41, 42, 46, 49, 50, 73, 74, 75, 76, 77, 78, 79, 89, 90, 91, 92, 93,
      94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117,
      118, 119, 120, 121, 123, 124, 125, 126, 127, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185,
      186, 187, 189, 190, 197, 223,
    );

    if ($oct != 4) return false;

    for ($i = 0; $i < $oct; $i++) {
      $iptest[$i] = trim($iptest[$i]);

      if ((!preg_match(":^[0-9]{1,3}$:", $iptest[$i])) || ($iptest[$i] > 255)) return false;
    }

    if (($iptest[0] < 1) || ($iptest[0] > 223) || ($iptest[3] < 1) || ($iptest[3] > 254)) return false;

    // loopback and non public addresses are only relevant for proxy checking
    if ($prx) {
      if ((in_array($iptest[0], $reserved)) ||
          (($iptest[0] == 169) && ($iptest[1] == 254)) ||
          (($iptest[0] == 172) && ($iptest[1] > 15) && ($iptest[1] < 32)) ||
          (($iptest[0] == 192) && ($iptest[1] === 0) && ($iptest[2] == 2)) ||
          (($iptest[0] == 192) && ($iptest[1] == 168)) ||
          (($iptest[0] == 198) && ($iptest[1] > 17) && ($iptest[1] < 20))) {
        return false;
      }
    }
    return (defined("_rev") ? implode(".", $iptest) : $srv);
  }

  // converts a hexadecimal IP to IP4 if applicable
  function bbc_hex2ip($str) {
    if (!preg_match(":[a-fA-F0-9]{8}:", $str)) return $str;

    $arr = explode(".", wordwrap($str, 2, ".", 2));

    for ($i = 0, $k = count($arr); $i < $k; $i++) $arr[$i] = trim(hexdec($arr[$i]));
    return ($arr[0].".".$arr[1].".".$arr[2].".".$arr[3]);
  }

  // returns the first valid host
  function bbc_select_host($array) {
    arsort($array, SORT_NUMERIC);

    foreach ($array as $key => $val) {
      $key = $this->bbc_hex2ip(trim($key));

      if (($prx = $this->bbc_valid_ip($key, 1)) !== false) return $prx;
    }
    return false;
  }

  // extract the first valid address from a chain
  function bbc_unchain_addr($val) {
    if (strpos($val, ",") === false) return $val;

    $array = explode(",", $val);

    for ($i = 0, $max = count($array); $i < $max; $i++) $array[$i] = trim($array[$i]);
    return $this->bbc_select_host(array_flip($array));
  }

  // return the correct remote address
  function bbc_get_remote_addr($addr, $reverse) {
    $addr = $this->bbc_unchain_addr($addr);
    $reverse = $this->bbc_unchain_addr($reverse);

    if ((!empty($reverse)) && ($this->bbc_valid_ip($addr, 1) === false)) return $reverse;
    elseif (empty($addr)) return "127.0.0.1";
    else return ((substr($addr, 0, strpos($addr, ".")) == 127) ? "127.0.0.1" : $addr);
  }

  // check for client in proxy headers
  function bbc_parse_headers() {
    reset($_SERVER);

    while (list($key, $val) = each($_SERVER)) {
      if (!(substr($key, 0, strpos($key, "_")) == "HTTP")) continue;

      if ((stristr($val, " for ") !== false)) {
        $tmp = explode(" for ", strtolower($val));
        $tmpval = trim($tmp[count($tmp) - 1]);
        $tmpval = $this->bbc_unchain_addr($tmpval);
        $chk[$tmpval] = isset($chk[$tmpval]) ? ++$chk[$tmpval] : 1;
      }
      if ((strpos($key, "_CLIENT") !== false) || (substr($key, -4) == "_FOR")) {
        $val = $this->bbc_unchain_addr($val);
        $chk[$val] = isset($chk[$val]) ? ++$chk[$val] : 1;
      }
      // If we find this, the Client IP is in reverse order and needs to be swapped
      if (($key == "HTTP_VIA") && (eregi("Traffic[ \-]?Server/(5\.([0-1]\.|2\.0))", $val))) {
        !defined("_rev") ? define("_rev", 1) : "";
      }
    }
    return (!empty($chk) ? $this->bbc_select_host($chk) : false);
  }

  // Check if an IP is matching up against the blacklist
  function bbc_is_ignored($blacklist, $client) {
    $ipmatch = (empty($blacklist) ? "" : explode(",", $blacklist));

    if (empty($ipmatch)) return false;

    for($i = count($ipmatch) - 1; $i >= 0; $i--) {
      $test = trim($ipmatch[$i]);
      if (substr($client, 0, strlen($test)) === $test) return true;
    }
    return false;
  }

  // checking for matching hosts which we have to ignore
  function bbc_ignore_ref($host, $list) {
    if (!empty($list)) {
      foreach(explode(",", $list) as $test) {
        if (strpos($host, trim($test)) !== false) return true;
      }
    }
    return false;
  }

  // checks for a valid url format
  function bbc_valid_ref($ref) {
    $tmp = explode(":", $ref);

    for ($i = 0, $k = count($tmp); $i < $k; $i++) $tmp[$i] = trim($tmp[$i]);
    return (((($tmp[0] == "http") || ($tmp[0] == "https")) && (substr($tmp[1], 0, 2) == "//")) ? true : false);
  }

  //converts a referrer to an array with the hostname, ip address and the full referrer
  function bbc_parse_ref($ref) {
    global $BBC_NO_DNS;

    if (!$this->bbc_valid_ref($ref)) return false;

    $test = substr(strstr($ref, "://"), 3);
    $host_raw = strtolower((($slash = strpos($test, "/")) !== false ? substr($test, 0, $slash) : $test));
    $host = (($port = strpos($host_raw, ":")) !== false ? substr($host_raw, 0, $port) : $host_raw);
    $uri = ($slash !== false ? substr($test, $slash) : "/");
    $uri = preg_replace(":[.]+(/|$):", "\\1", preg_replace(":[/]+:", "/", $uri));
    $addr = empty($BBC_NO_DNS) ? gethostbyname($host) : $host;

    return array("http://".$host_raw.$uri, $host, $addr);
  }

  // determine and filter stuff which came from the local server
  function bbc_filter_ref($srvhost, $ref, $srvname, $srvaddr) {
    global $BBC_IGNORE_REFER, $BBC_NO_DNS;

    if ((!($ref_array = $this->bbc_parse_ref($ref))) ||
        ($this->bbc_ignore_ref($ref_array[1], $BBC_IGNORE_REFER) !== false)) {
      return;
    }

    if (((!empty($srvhost)) && ($srvhost == $ref_array[1])) || ($ref_array[1] == $srvaddr) ||
        ($ref_array[1] == $srvname) || ((substr($srvname, 0, 4) == "www.") && (substr($srvname, 4) == $ref_array[1])) ||
        (empty($BBC_NO_DNS) ? ($ref_array[1] == gethostbyaddr($srvaddr)) : "")) {
      return;
    }

    $ref = $ref_array[0];

    return (((substr($ref_array[2], 0, 4) == "127.") || (substr($ref_array[2], 0, 2) == "0.")) ? "" : $ref);
  }

  // avoid trails of query strings which aren't relevant for page counting
  function bbc_filter_uri($script, $pinfo, $uri) {
    // getting rid of stupid user input
    foreach (array("pinfo", "uri") as $path) {
      ${$path} = preg_replace(":[/]+:", "/", preg_replace(":[.]+(/|$):", "\\1", ${$path}));
    }

    // On some systems path info is just an alias for the script uri
    $pinfo = ($uri == $pinfo) ? 0 : $pinfo;

    $uri = !empty($pinfo) ? substr($uri, 0, (strlen($uri) - strlen($pinfo))) : $uri;
    $uri = (basename($uri) !== $script) ? (((($dir = dirname($uri)) == ".") || (empty($dir))) ? "/" : $dir."/")
           .$script : $uri;

    $test = explode(".", $script);
    $tmp = strtolower(trim($test[0]));
    $tmp = ((count($test) == 2) && (($tmp == "index") || ($tmp == "default"))) ? true : false;

    return (($tmp !== false) ? substr($uri, 0, (strrpos($uri, "/") + 1)) : (empty($uri) ? "/" : $uri));
  }

  // automatic page name generation in case of not being specified
  function bbc_auto_page_name($uri) {
    if (!is_string($uri) || ($uri == "/")) return "index";

    $uri = (strlen($uri) - 1 === ($slash = strrpos($uri, "/"))) ? substr($uri, 1, --$slash) :
           (($dot = strrpos($uri, ".")) !== false ? substr($uri, 1, --$dot) : substr($uri, 1));
    $uri = str_replace("/", " -&gt; ", str_replace("_", " ", $uri));

    return ucwords($uri);
  }

  // write the entry
  function bbc_write_entry() {
    global $BBC_CACHE_PATH, $BBC_NO_STRING;

    $file = $this->filename;
    $base = basename($file);

    if (!is_readable($file)) return array($base, "err_acc");
    if (!is_writable($file)) return array($base, "err_wri");

    $fp = fopen($file, "a+");

    if (flock($fp, LOCK_EX)) {
      fputs($fp, $this->string);
      fflush($fp);
      flock($fp, LOCK_UN);

      $ok = 1;
    }
    fclose($fp);

    return (isset($ok) ? array($base, "ok") : array($base, "err_lock"));
  }

  // constructor
  function bbc_marker() {
    global $BBC_CACHE_PATH, $BBC_COUNTER_PREFIX, $BBC_COUNTER_SUFFIX, $BBC_IGNORE_IP, $BBC_NO_DNS, $BBC_SEP,
           $BBC_TIMESTAMP, $BBC_TIME_OFFSET;

    $this->sep = $BBC_SEP;
    $this->ignored = false;
    $this->filename = $this->bbc_counter_file($BBC_CACHE_PATH, $BBC_COUNTER_PREFIX, $BBC_COUNTER_SUFFIX);
    $time = $BBC_TIMESTAMP + ($BBC_TIME_OFFSET * 60);

    // loads of initialisations
    $hdr = array("HTTP_USER_AGENT", "LOCAL_ADDR", "REMOTE_HOST", "REMOTE_ADDR", "HTTP_HOST", "HTTP_REFERER",
                 "HTTP_X_REMOTECLIENT_IP", "PATH_INFO", "PATH_TRANSLATED", "HTTP_PC_REMOTE_ADDR", "PHP_SELF",
                 "SCRIPT_FILENAME", "SERVER_NAME", "SERVER_ADDR");

    foreach($hdr as $str) $$str = !empty($_SERVER[$str]) ? $this->bbc_clean($_SERVER[$str]) : "";

    $filename = empty($PATH_TRANSLATED) ? basename($SCRIPT_FILENAME) : basename($PATH_TRANSLATED);
    $REQUEST_URI = $this->bbc_filter_uri($filename, $PATH_INFO , $PHP_SELF);
    $SERVER_ADDR = empty($SERVER_ADDR) ? $LOCAL_ADDR : $SERVER_ADDR;
    $SERVER_ADDR = $this->bbc_valid_ip($SERVER_ADDR) ? $SERVER_ADDR : "127.0.0.1";
    $HTTP_USER_AGENT = empty($HTTP_USER_AGENT) ? "unknown" : $HTTP_USER_AGENT;
    $HTTP_REFERER = $this->bbc_filter_ref($HTTP_HOST, $HTTP_REFERER, $SERVER_NAME, $SERVER_ADDR);
    $HTTP_REFERER = empty($HTTP_REFERER) ? "unknown" : $HTTP_REFERER;
    // Use a page name even if the user didn't specify it
    $page = defined("_BBC_PAGE_NAME") ? $this->bbc_clean(_BBC_PAGE_NAME) : $this->bbc_auto_page_name($REQUEST_URI);
    $prx = $this->bbc_parse_headers();
    $reverse = !empty($HTTP_PC_REMOTE_ADDR) ? $HTTP_PC_REMOTE_ADDR : $HTTP_X_REMOTECLIENT_IP;

    if (!empty($prx)) {
      $prx_addr = $this->bbc_get_remote_addr($REMOTE_ADDR, $reverse);
      $REMOTE_ADDR = $this->bbc_clean($prx);
      $REMOTE_HOST = empty($BBC_NO_DNS) ? $this->bbc_clean(gethostbyaddr($prx)) : $REMOTE_ADDR;
    }
    else {
      $prx_addr = "unknown";
      $REMOTE_ADDR = $this->bbc_get_remote_addr($REMOTE_ADDR, $reverse);
      $REMOTE_HOST = empty($BBC_NO_DNS) ? $this->bbc_clean(gethostbyaddr($REMOTE_ADDR)) : $REMOTE_ADDR;
    }

    // Stop processing ignored adresses
    if (($this->ignored = $this->bbc_is_ignored($BBC_IGNORE_IP, $REMOTE_ADDR)) !== false) return;

    $this->string = $time.$this->sep.$prx_addr.$this->sep.$REMOTE_ADDR.$this->sep.$REMOTE_HOST.$this->sep
                   .$HTTP_USER_AGENT.$this->sep.$HTTP_REFERER.$this->sep.$REQUEST_URI.$this->sep.$page."\n";
  }
}
?>